/**
 * MEDOR: Middleware Enabling Distributed Object Requests
 * 
 * Copyright (C) 2001-2003 France Telecom R&D
 * Contact: alexandre.lefebvre@rd.francetelecom.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Initial developers: M. Alia, A. Lefebvre
 */
package org.objectweb.medor.expression.lib;

import org.objectweb.jorm.type.api.PTypeSpace;
import org.objectweb.jorm.type.api.PType;
import org.objectweb.medor.expression.api.Expression;
import org.objectweb.medor.expression.api.ExpressionException;
import org.objectweb.medor.expression.api.MalformedExpressionException;
import org.objectweb.medor.expression.api.Operand;
import org.objectweb.medor.expression.api.Operator;
import org.objectweb.medor.expression.api.ParameterOperand;
import org.objectweb.medor.expression.api.TypingException;
import org.objectweb.medor.expression.api.UnaryOperator;

public class Length extends BasicUnaryOperator implements UnaryOperator {

    public Length() {
    }

    public Length(Expression e) {
        this.operand = e;
    }

    public org.objectweb.medor.expression.api.Operand evaluate(ParameterOperand[] pos, Object o)
            throws ExpressionException {
        try {
            result.setValue(evaluate(operand.evaluate(pos, o).getString()));
        } catch (NullPointerException e) {
            throw new ExpressionException("Unevaluable Expression: Not compiled");
        }
		return result;
    }

    public int evaluate(String op) {
        return op.length();
    }

    public Operand compileExpression()
            throws ExpressionException, MalformedExpressionException {
        if (operand != null) {
            operand.compileExpression();
            if (operand.getType().getTypeCode() == PType.TYPECODE_STRING) {
                type = PTypeSpace.INT;
                result = new BasicVariableOperand(type);
                verified = true;
            } else
                throw new TypingException("Attempt a String type");
        } else
			//Operands not yet assigned
            throw new MalformedExpressionException("null childeren value");
		return result;
    }

    public String getOperatorString() {
        return Operator.LENGTH;
    }
}
