/**
 * MEDOR: Middleware Enabling Distributed Object Requests
 *
 * Copyright (C) 2001-2003 France Telecom R&D
 * Contact: alexandre.lefebvre@rd.francetelecom.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Initial developers: M. Alia, A. Lefebvre, S. Chassande-Barrioz
 */
package org.objectweb.medor.expression.lib;

import org.objectweb.jorm.type.api.PType;
import org.objectweb.jorm.type.api.PTypeSpace;
import org.objectweb.medor.expression.api.Expression;
import org.objectweb.medor.expression.api.ExpressionException;
import org.objectweb.medor.expression.api.MalformedExpressionException;
import org.objectweb.medor.expression.api.Operand;
import org.objectweb.medor.expression.api.Operator;
import org.objectweb.medor.expression.api.ParameterOperand;

/**
 * @author S.Chassande-Barrioz
 */
public class Like implements Operator {

	public String getOperatorString() {
		return (not ? "NOT LIKE" : "LIKE");
	}

	Expression[] param;
	boolean not = false;

	public Like(Expression str, Expression pattern, Expression escape) {
		param = new Expression[] {str, pattern, escape};
	}

	public Like(Expression str, Expression pattern, Expression escape, boolean not) {
		param = new Expression[] {str, pattern, escape};
		this.not = not;
	}

	public Like(Expression str, Expression pattern) {
		param = new Expression[] {str, pattern};
	}

	public Like(Expression str, Expression pattern, boolean not) {
		param = new Expression[] {str, pattern};
		this.not = not;
	}

	// IMPLEMENTATION OF THE Operator INTERFACE //
	//------------------------------------------//

	public PType getType() {
		return PTypeSpace.BOOLEAN;
	}

	public int getOperandNumber() {
		return param.length;
	}

	public Expression getExpression(int idx) throws ArrayIndexOutOfBoundsException {
		return param[idx];
	}

	public Operand compileExpression()
		throws ExpressionException, MalformedExpressionException {
		throw new ExpressionException("Not yet implemented");
	}

	public void setExpression(int idx, Expression exp) throws ArrayIndexOutOfBoundsException {
		param[idx] = exp;
	}

	public Operand getResult() throws IllegalStateException {
		return null;
	}

	public org.objectweb.medor.expression.api.Operand
			evaluate(ParameterOperand[] values, Object o)
		throws ExpressionException {
		//TODO Implement Like operator
		throw new ExpressionException("Not yet implemented");
	}
}
